import pytest
import xarray as xr
import pandas as pd
import numpy as np
from matplotlib.axes import Axes


from loadest_gp.providers import usgs
from loadest_gp import LoadestGPMarginalGPyTorch as LoadestGP
from loadest_gp.utils import concentration_to_flux, plot_annual_flux
from discontinuum.utils import time_substitution, aggregate_to_daily


@pytest.fixture
def sample_dataset():
    # Create a sample dataset for testing
    n = 20
    times = pd.date_range("2000-01-01", "2010-12-31", periods=n)
    flow = np.exp(np.random.randn(n))  # Random streamflow data
    concentration = np.exp(np.random.randn(n)) # Random solute concentration data

    ds = xr.Dataset(
        data_vars={
            'flow': ('time', flow),
            'concentration': ('time', concentration),
        },
        coords={'time': times},
    )

    return ds


@pytest.fixture
def daily_dataset():
    times = pd.date_range("2000-01-01", "2010-12-31", freq='d')
    flow = np.exp(np.random.rand(len(times)))  # Random streamflow data

    ds = xr.Dataset(
        data_vars={'flow': ('time', flow)},
        coords={'time': times},
      )
    
    return ds


def test_aggregate_to_daily():
    dates_hourly = pd.date_range('2000-01-01', '2000-02-01', freq='h')
    ds = xr.Dataset(data_vars={'data': ('time', np.ones(len(dates_hourly)))},
                    coords={'time': dates_hourly},)
    ds = aggregate_to_daily(ds)

    dates_daily = pd.date_range('2000-01-01', '2000-02-01', freq='d')

    assert all(ds.data == 1)
    assert all(ds.time == dates_daily)


def test_concentration_to_flux():
    dates = pd.date_range('2000-01-01', '2000-02-01', freq='d')
    ds = xr.Dataset(data_vars={'concentration': ('time', np.ones(len(dates))),
                               'flow': ('time', np.ones(len(dates)))},
                    coords={'time': dates})
    ds['concentration'].attrs['units'] = "mg/l"
    ds['concentration'].attrs['long_name'] = "Concentration"
    ds['flow'].attrs['units'] = "cubic meters per second"
    
    flux = concentration_to_flux(ds['concentration'], ds['flow'])

    assert isinstance(flux, xr.DataArray)
    assert hasattr(flux, 'time')
    assert all(flux.time == ds.time)
    
    assert isinstance(plot_annual_flux(flux), Axes)


def test_loadest_gp(sample_dataset):

    #data = sample_dataset()

    model = LoadestGP()
    model.fit(target=sample_dataset['concentration'],
              covariates=sample_dataset[['time','flow']],
              iterations=10)

    assert model.is_fitted
    assert isinstance(model.contourf(levels=5, y_scale='log'), Axes)  


def test_time_substitution():
    ds = xr.Dataset(data_vars={'data': ('time', np.linspace(0, 1, 10))},
                    coords={'time': np.arange(10)})
    slice_idx = np.random.randint(0, 10)
    ds_sub = time_substitution(ds, interval=slice(slice_idx, slice_idx))

    assert all(ds_sub.data == ds.data.isel(time=slice_idx).values)
    assert all(ds_sub.time == ds.time)



import pytest
import numpy as np
import pandas as pd
import xarray as xr
from matplotlib.axes import Axes
from matplotlib.colorbar import Colorbar

from rating_gp.providers import usgs
from rating_gp.models.gpytorch import RatingGPMarginalGPyTorch as RatingGP


@pytest.fixture
def training_data():
    # Create a sample dataset for testing
    n = 20
    times = pd.date_range("2000-01-01", "2010-12-31", periods=n)
    discharge = np.exp(np.random.randn(n))  # Random streamflow data
    discharge_unc = 1 + 0.1 * np.random.rand(n)  # Random uncertainty data
    stage = np.random.randn(n) # Random stage data

    ds = xr.Dataset(
        data_vars={
            'discharge': ('time', discharge),
            'discharge_unc': ('time', discharge_unc),
            'stage': ('time', stage),
        },
        coords={'time': times},
    )

    return ds


def test_rating_gp(training_data):

    model = RatingGP()
    model.fit(target=training_data['discharge'],
              covariates=training_data[['stage']],
              target_unc=training_data['discharge_unc'],
              iterations=10)
    assert model.is_fitted

    assert isinstance(model.plot_stage(), Axes)
    assert isinstance(model.plot_discharge(), Axes)
    assert isinstance(model.plot_observed_rating(), Axes)
    assert isinstance(model.plot_rating(covariates=training_data[['stage']]), Axes)
    ax = model.plot_ratings_in_time(
        time=pd.date_range('1990', '2021', freq='5YS-OCT'), ci=0.95
    )
    assert isinstance(ax, Axes)
    assert isinstance(model.add_time_colorbar(ax=ax), Colorbar)



# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

project = 'discontinuum'
author = 'Timothy Hodson and Keith Doore'
release = 'v0.5'

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration
import os
os.environ["TQDM_DISABLE"] = "True"

extensions = [
    "myst_nb",
    "sphinxcontrib.mermaid",
    'sphinx.ext.autodoc',
    "sphinx.ext.autosummary",
    "sphinx.ext.githubpages",
    "sphinx.ext.napoleon",
    "sphinx_copybutton",
    ]

autosummary_generate = True
autodoc_default_options = {
       'members': True,
   }
templates_path = ['_templates']
exclude_patterns = []
nb_execution_timeout = -1


# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = 'sphinx_book_theme'
# html_static_path = ['_static']
